/*******************************************************************************
*                                                                              *
* foo_dsp_centercut                                                            *
* Copyright (C) 2006  Moitah (moitah@yahoo.com)                                *
*                                                                              *
* This program is free software; you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation; either version 2 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program; if not, write to the Free Software                  *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA    *
*                                                                              *
*******************************************************************************/

#include "foo_dsp_centercut.h"

DECLARE_COMPONENT_VERSION(
	"Center Cut",
	"1.0.1",
	"foo_dsp_centercut\n"
	"Copyright 2006 Moitah\n"
	"http://www.moitah.net/\n\n"
	"Based on VirtualDub's Center Cut filter by Avery Lee."
	);

class foo_dsp_centercut_sides : public dsp_impl_base {
private:
	CenterCut *_cc;
	int _sampleRate;
	int _bufferedSamples;

public:
	foo_dsp_centercut_sides() {
		_cc = new CenterCut(false);
		_bufferedSamples = 0;
	}

	~foo_dsp_centercut_sides() {
		delete _cc;
	}

	static GUID g_get_guid() {
		static const GUID guid = { 0xd72fc836, 0x56d2, 0x4c74, { 0x95, 0x7e, 0xf0, 0xb6, 0xac, 0x80, 0xe0, 0x79 } };
		return guid;
	}

	static void g_get_name(pfc::string_base & p_out) {
		p_out = "Center Cut - Sides";
	}

	virtual bool on_chunk(audio_chunk * inChunk, abort_callback &) {
		_sampleRate = inChunk->get_sample_rate();
		if (inChunk->get_channels() != 2) {
			return true;
		}

		const audio_sample *inSamples = inChunk->get_data();
		int inSampleCount = inChunk->get_sample_count();

		while (inSampleCount > 0) {
			int copyCount = min(inSampleCount, _cc->GetInputSamplesNeeded());
			double *ccInput = _cc->GetInputBuffer();

			for (int i = 0; i < copyCount; i++) {
				ccInput[0] = (double)inSamples[0];
				ccInput[1] = (double)inSamples[1];
				ccInput += 2;
				inSamples += 2;
			}
			inSampleCount -= copyCount;
			_bufferedSamples += copyCount;

			if (_cc->ProcessSamples(copyCount)) {
				OutputSamples(_cc->GetOutputBuffer(), _cc->GetOutputLength());
			}
		}

		return false;
	}

	virtual void flush() {
		_cc->Flush();
		_bufferedSamples = 0;
	}

	virtual void on_endoftrack(abort_callback &) {
		FlushToOutput();
	}

	virtual void on_endofplayback(abort_callback &) {
		FlushToOutput();
	}

	virtual double get_latency() {
		return ((double)_bufferedSamples / (double)_sampleRate);
	}

	virtual bool need_track_change_mark() {
		return false;
	}

private:
	void FlushToOutput() {
		int outSampleCount;

		while ((outSampleCount = _cc->FlushWithReturn()) != -1) {
			if (outSampleCount > 0) {
				OutputSamples(_cc->GetOutputBuffer(), outSampleCount);
			}
		}
	}

	void OutputSamples(double *ccOutput, int outSampleCount) {
		audio_chunk *outChunk;
		audio_sample *outSamples;
		
		outChunk = insert_chunk(outSampleCount);
		outChunk->set_data_size(outSampleCount * 2);
		outChunk->set_sample_count(outSampleCount);
		outChunk->set_channels(2);
		outChunk->set_sample_rate(_sampleRate);
		outSamples = outChunk->get_data();

		for (int i = 0; i < outSampleCount; i++) {
			outSamples[0] = (audio_sample)ccOutput[0];
			outSamples[1] = (audio_sample)ccOutput[1];
			outSamples += 2;
			ccOutput += 2;
		}

		_bufferedSamples -= outSampleCount;
	}
};

class foo_dsp_centercut_center : public dsp_impl_base {
private:
	CenterCut *_cc;
	int _sampleRate;
	int _bufferedSamples;

public:
	foo_dsp_centercut_center() {
		_cc = new CenterCut(true);
		_bufferedSamples = 0;
	}

	~foo_dsp_centercut_center() {
		delete _cc;
	}

	static GUID g_get_guid() {
		static const GUID guid = { 0xf922c36f, 0xec93, 0x49f5, { 0xab, 0x98, 0x57, 0x16, 0xd4, 0x24, 0x11, 0xcd } };
		return guid;
	}

	static void g_get_name(pfc::string_base & p_out) {
		p_out = "Center Cut - Center";
	}

	virtual bool on_chunk(audio_chunk * inChunk, abort_callback &) {
		_sampleRate = inChunk->get_sample_rate();
		if (inChunk->get_channels() != 2) {
			return true;
		}

		const audio_sample *inSamples = inChunk->get_data();
		int inSampleCount = inChunk->get_sample_count();

		while (inSampleCount > 0) {
			int copyCount = min(inSampleCount, _cc->GetInputSamplesNeeded());
			double *ccInput = _cc->GetInputBuffer();

			for (int i = 0; i < copyCount; i++) {
				ccInput[0] = (double)inSamples[0];
				ccInput[1] = (double)inSamples[1];
				ccInput += 2;
				inSamples += 2;
			}
			inSampleCount -= copyCount;
			_bufferedSamples += copyCount;

			if (_cc->ProcessSamples(copyCount)) {
				OutputSamples(_cc->GetOutputBuffer(), _cc->GetOutputLength());
			}
		}

		return false;
	}

	virtual void flush() {
		_cc->Flush();
		_bufferedSamples = 0;
	}

	virtual void on_endoftrack(abort_callback &) {
		FlushToOutput();
	}

	virtual void on_endofplayback(abort_callback &) {
		FlushToOutput();
	}

	virtual double get_latency() {
		return ((double)_bufferedSamples / (double)_sampleRate);
	}

	virtual bool need_track_change_mark() {
		return false;
	}

private:
	void FlushToOutput() {
		int outSampleCount;

		while ((outSampleCount = _cc->FlushWithReturn()) != -1) {
			if (outSampleCount > 0) {
				OutputSamples(_cc->GetOutputBuffer(), outSampleCount);
			}
		}
	}

	void OutputSamples(double *ccOutput, int outSampleCount) {
		audio_chunk *outChunk;
		audio_sample *outSamples;
		
		outChunk = insert_chunk(outSampleCount);
		outChunk->set_data_size(outSampleCount * 1);
		outChunk->set_sample_count(outSampleCount);
		outChunk->set_channels(1);
		outChunk->set_sample_rate(_sampleRate);
		outSamples = outChunk->get_data();

		for (int i = 0; i < outSampleCount; i++) {
			outSamples[0] = (audio_sample)ccOutput[0];
			outSamples += 1;
			ccOutput += 2;
		}

		_bufferedSamples -= outSampleCount;
	}
};

static dsp_factory_nopreset_t<foo_dsp_centercut_sides> df_foo_dsp_centercut_sides;
static dsp_factory_nopreset_t<foo_dsp_centercut_center> df_foo_dsp_centercut_center;